#ifndef KYSEC_TEE_ALGO_H__
#define KYSEC_TEE_ALGO_H__

#include <kysec_tee_def.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 生成sm2公私钥对
 * 
 * @param pub_key 输出，sm2公钥
 * @param pri_key 输出，sm2私钥
 * @return 0 成功， 其他 失败
 */
extern KT_Result kysec_tee_algo_sm2_keygen(uint8_t pub_key[64], uint8_t pri_key[32]);

/**
 * @brief sm2数据签名
 * 
 * @param data 需签名数据
 * @param pub_key sm2公钥
 * @param pri_key sm2私钥
 * @param sign_data 传出，签名数据
 * @return 0 成功， 其他失败
 */
extern KT_Result kysec_tee_algo_sm2_sign(KYSEC_TEE_DATA data, const uint8_t pub_key[64], 
        const uint8_t pri_key[32], uint8_t sign_data[64]);

/**
 * @brief sm2数据验签
 * 
 * @param data 待验签数据
 * @param pub_key sm2公钥
 * @param sign_data 签名数据
 * @return 0 验签通过， 1 验签失败， 其他接口调用失败
 */
extern KT_Result kysec_tee_algo_sm2_verify(KYSEC_TEE_DATA data, const uint8_t pub_key[64], uint8_t sign_data[64]);

/**
 * @brief sm2数据加密
 * 
 * @param data 待加密数据，最大数据长度4000字节
 * @param pub_key sm2公钥
 * @param out_data 加密完成数据， dptr指向内存接口内部申请，使用完成后，需外部释放
 * @return 0 成功，其他失败 
 */
extern KT_Result kysec_tee_algo_sm2_encrypt(KYSEC_TEE_DATA data, const uint8_t pub_key[64], KYSEC_TEE_DATA *out_data);

/**
 * @brief sm2数据解密
 * 
 * @param data 待解密数据，最大长度4096字节
 * @param pub_key sm2公钥
 * @param pri_key sm2私钥
 * @param out_data 解密数据， dptr指向内存接口内部申请，使用完成后，需外部释放
 * @return 0 成功，其他失败
 */
extern KT_Result kysec_tee_algo_sm2_decrypt(KYSEC_TEE_DATA data, const uint8_t pub_key[64], 
        const uint8_t pri_key[32], KYSEC_TEE_DATA *out_data);

/**
 * @brief 初始化sm3上下文
 * 
 * @param context sm3上下文，内存接口内部申请，外部使用完成后释放
 * @return 0 成功，其他失败
 */
extern KT_Result kysec_tee_algo_sm3_init(ALGO_SM3_CTX **context);

/**
 * @brief sm3数据计算
 * 
 * @param context sm3上下文，kysec_tee_algo_sm3_init初始化
 * @param data 待进行摘要计算数据，最大长度4096字节
 * @return 0 成功，其他失败 
 */
extern KT_Result kysec_tee_algo_sm3_update(ALGO_SM3_CTX *context, KYSEC_TEE_DATA data);

/**
 * @brief sm3摘要计算完成
 * 
 * @param context sm3上下文，kysec_tee_algo_sm3_init初始化
 * @param digest 传出，摘要计算结果
 * @return 0 成功，其他失败 
 */
extern KT_Result kysec_tee_algo_sm3_finalize(ALGO_SM3_CTX *context, uint8_t digest[32]);

/**
 * @brief 初始化sm4上下文
 * 
 * @param context sm4上下文，内存接口内部申请，外部使用完成后释放
 * @param mode 计算模式，ECB 或 CBC
 * @param type 计算类型，加密或解密
 * @param key 密钥
 * @param iv 向量，CBC模式使用
 * @return 0成功，其他失败 
 */
extern KT_Result kysec_tee_algo_sm4_init(ALGO_SM4_CTX **context, ALGO_SM4_OP_MODE mode, ALGO_SM4_OP_TYPE type,
        const uint8_t key[16], const uint8_t iv[16]);

/**
 * @brief sm4执行加解密操作
 * 
 * @param context sm3上下文，kysec_tee_algo_sm4_init初始化
 * @param type 计算类型，加密或解密
 * @param data 需加解密数据，最大长度4096, 解密或非最后帧加密时，数据长度必须是16倍数，
 * @param flag 是否最后一帧， 0 非最后帧， 1 最后帧
 * @param out_data 加解密数据结果，dptr指向内存接口内部申请，使用完成后，需外部释放
 * @return 0成功，其他失败
 * @attention 加密时数据采用PKCS7填充，最后帧数据解密结果可能为空
 */
extern KT_Result kysec_tee_algo_sm4_update(ALGO_SM4_CTX *context, ALGO_SM4_OP_TYPE type, KYSEC_TEE_DATA data, 
        const uint32_t flag, KYSEC_TEE_DATA *out_data);

/**
 * @brief 获取随机数
 * 
 * @param len 获取随机数长度
 * @param data 传出随机数，dptr指向内存接口内部申请，使用完成后，需外部释放
 * @return 0成功，其他失败 
 */
extern KT_Result kysec_tee_algo_random(uint32_t len, KYSEC_TEE_DATA *data);

#ifdef __cplusplus
}
#endif

#endif